//
//  AuthV4ViewController
//  HIVE_SDK_Sample
//
//  Created by jhjang on 03/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum AuthV4Api: String, CaseIterable {
    case setup = "Setup"
    case signIn = "SignIn"
    case showSignIn = "Show SignIn"
    case signOut = "SignOut"
    case connect = "Connect"
    case disconnect = "Disconnect"
    case isAutoSignIn = "Is Auto SignIn"
    case getPlayerInfo = "Get Player Info"
    case selectConflict = "Select Conflict"
    case getProviderFriendsList = "Get Friends"
    case showConflict = "Show Conflict"
    case showProfile = "Show Profile"
    case checkMaintenance = "Check Maintenance"
    case checkBlacklist = "Check Blacklist"
    case setProviderChangeListener = "Set Provider Change Listener"
	case showInquiry = "show Inquiry"
    case showChatbot = "show Chatbot"
	case showTerms = "Show Terms"
    case reset = "Reset"
    case showAdultConfirm = "Show Adult Confirm"
    case showGameCenterPopup = "GameCenter Popup"
    case configurationGetPermissionViewData = "Configuration getPermissionViewData"
    case authPermissionViewData = "AuthV4 getPermissionViewData"
    case providerAppleReportScore = "GameCenter ReportScore"
    case providerAppleReportAchievement = "GameCenter Report Achievement"
    case providerAppleShowLeaderBoard = "GameCenter Show LeaderBoard"
    case providerAppleShowAchievements = "GameCenter Show Achievements"
    case providerAppleLoadAchievements = "GameCenter Load Achievements"
    case providerAppleResetAchievements = "GameCenter Reset Achievements"
}

enum AuthV4IDPType {
    case signIn
    case connect
    case disconnect
    case friends
}

class AuthV4ViewController: BaseViewController {
    let apis: [AuthV4Api] = AuthV4Api.allCases
    var providerList: [ProviderType]?
    var actionSheetType: AuthV4IDPType = .signIn
    var conflictPlayer: PlayerInfo?
    
    var actionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Provider", message: "Select Provider", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        if let providerList = self.providerList {
            for item in providerList {
                let actionStyle:UIAlertAction.Style = item == .Auto ? .destructive : .default
                
                let action = UIAlertAction(title: item.description, style: actionStyle) { (action) in
                    switch self.actionSheetType {
                    case .connect:
                        AuthV4Interface.connect(item) { (result, playerInfo) in
                            let log = """
                            === AuthV4Interface.connect() ===
                            
                            result - \(result)
                            playerInfo - \(playerInfo)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.getErrorCode() == .conflictPlayer {
                                self.conflictPlayer = playerInfo
                            }
                        }
                    case .signIn:
                        AuthV4Interface.signIn(item) { (result, playerInfo) in
                            let log = """
                            === AuthV4Interface.signIn() ===
                            
                            result - \(result)
                            playerInfo - \(playerInfo)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        }
                        
                    case .disconnect:
                        AuthV4Interface.disconnect(item, handler: { (result) in
                            let log = """
                            === AuthV4Interface.disconnect() ===
                            
                            result - \(result)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        })
                    
                    case .friends:
                        AuthV4Interface.getProviderFriendsList(item, handler: { (result, providerType, friendsList) in
                            let log = """
                            === AuthV4Interface.getProviderFriendsList() ===
                            
                            result - \(result)
                            providerType - \(providerType)
                            friendsList - \(friendsList)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        })
                    }
                }
                actionSheet.addAction(action)
            }
        }
        
        return actionSheet
    }
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: AuthV4Api) {
        switch api {
        case .setup:
            AuthV4Interface.setup { (result, isAutoSignIn, did, providerList) in
                let log = """
                === AuthV4Interface.setup() ===
                
                result - \(result)
                isAutoSignIn - \(isAutoSignIn)
                did - \(did)
                providerList - \(providerList)
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.providerList = providerList?.compactMap { ProviderType(rawValue: $0) }
                
                // Test
                self.providerList?.append(.WeChat)
                self.providerList?.append(.VK)
                self.providerList?.append(.QQ)
                self.providerList?.append(.Line)
                self.providerList?.append(.Twitter)
                self.providerList?.append(.Guest)
                
                if isAutoSignIn {
                    self.providerList?.insert(.Auto, at: 0)
                }
            }
            
        case .signIn:
            self.actionSheetType = .signIn
            self.present(self.actionSheet, animated: true, completion: nil)
            
        case .signOut:
            AuthV4Interface.signOut { (result) in
                let log = """
                === AuthV4Interface.signOut() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
            
        case .connect:
            self.actionSheetType = .connect
            self.present(self.actionSheet, animated: true, completion: nil)
            
        case .disconnect:
            self.actionSheetType = .disconnect
            self.present(self.actionSheet, animated: true, completion: nil)
            
        case .isAutoSignIn:
            let isAutoSignIn = AuthV4Interface.isAutoSignIn()
            
            let log = """
            === AuthV4Interface.isAutoSignIn() ===
            
            isAutoSignIn - \(isAutoSignIn)
            """
            
            self.logDelegate?.addLog(log: log)
        case .getPlayerInfo:
            let playerInfo = AuthV4Interface.getPlayerInfo()
            
            let log = """
            === AuthV4Interface.getPlayerInfo() ===
            
            playerInfo - \(playerInfo)
            """
            
            self.logDelegate?.addLog(log: log)
            
        case .getProviderFriendsList:
            self.actionSheetType = .friends
            self.present(self.actionSheet, animated: true, completion: nil)
        case .selectConflict:
            self.showConflict()
        case .showSignIn:
            AuthV4Interface.showSignIn { (result, playerInfo) in
                let log = """
                === AuthV4Interface.showSignIn() ===
                
                result - \(result)
                playerInfo - \(playerInfo)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showProfile:
            AuthV4Interface.showProfile(AuthV4Interface.getPlayerInfo()?.playerID ?? -1) { (result) in
                let log = """
                === AuthV4Interface.showProfile() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showConflict:
            let currentUserData: [String:Any] = [
                "player_id" : AuthV4Interface.getPlayerInfo()?.playerID ?? -1,
                "game_data" : [
                    "Gold" : "1024",
                    "Gem" : "110"
                ]
            ]
            
            let conflictUserData: [String:Any] = [
                "player_id" : self.conflictPlayer?.playerID ?? -1,
                "game_data" : [
                    "Gold" : "2048",
                    "Gem" : "220"
                ]
            ]
            
            AuthV4Interface.showConflictSelection(currentUserData, conflictPlayerData: conflictUserData) { (result, playerInfo) in
                let log = """
                === AuthV4Interface.showConflictSelection() ===
                
                result - \(result)
                playerInfo - \(playerInfo)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        
        case .reset:
            AuthV4Interface.reset()
            
        case .checkMaintenance:
            AuthV4Interface.checkMaintenance(true) { (result, maintenances) in
                let log = """
                === AuthV4Interface.checkMaintenance() ===
                
                result - \(result)
                maintenances - \(maintenances)
                """
                
                self.logDelegate?.addLog(log: log)
            }
            
        case .checkBlacklist:
            AuthV4Interface.checkBlacklist(true) { (result, blacklist) in
                let log = """
                === AuthV4Interface.checkBlacklist() ===
                
                result - \(result)
                blacklist - \(blacklist)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .setProviderChangeListener:
            AuthV4Interface.setProviderChangedListener { (result, providerInfo) in
                let log = """
                === AuthV4Interface.setProviderChangedListener() ===
                
                result - \(result)
                providerInfo - \(providerInfo)
                """
                
                self.logDelegate?.addLog(log: log)
            }
		case .showInquiry:
			AuthV4Interface.showInquiry { (result) in
				let log = """
				=== AuthV4Interface.showInquiry() ===
				
				result - \(result)
				"""
				
				self.logDelegate?.addLog(log: log)
			}
        case .showChatbot:
            AuthV4Interface.showChatbotInquiry("{\"evt_code\":1000,\"p1\":\"random\"}") { (result) in
                let log = """
                === AuthV4Interface.showChatBotInquiry() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
		case .showTerms:
			AuthV4Interface.showTerms { (result) in
				let log = """
				=== AuthV4Interface.showTerms() ===
				
				result - \(result)
				"""
				
				self.logDelegate?.addLog(log: log)
			}
        case .showAdultConfirm:
            AuthV4Interface.showAdultConfirm { (result) in
                let log = "result - \(result)"
                self.logDelegate?.addLog(log: log)
            }
        case .showGameCenterPopup:
            AuthV4Interface.showGameCenterLoginCancelDialog { (result) in
                let log = """
                === AuthV4Interface.showGameCenterLoginCancelDialog() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .configurationGetPermissionViewData:
            let result = ConfigurationInterface.getPermissionViewData(.KO)
            self.logDelegate?.addLog(log: result.description)
        case .authPermissionViewData:
            AuthV4Interface.requestPermissionViewData { (result, data) in
                
                self.logDelegate?.addLog(log: data?.description ?? "")
            }
        case .providerAppleReportScore:
            ProviderApple.shared.reportScore("00", leaderboardIdentifier: "grp.com.com2us.hivesdk.10hit") { (result) in
                let log = """
                === ProviderApple.shared.reportScore() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .providerAppleReportAchievement:
            ProviderApple.shared.reportAchievement("100", showsCompletionBanner: true, achievementIdentifier: "grp.com.com2us.hivesdk.10hit") { (result) in
                let log = """
                === ProviderApple.shared.reportAchievement() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .providerAppleShowLeaderBoard:
            ProviderApple.shared.showLeaderboard { (result) in
                let log = """
                === ProviderApple.shared.showLeaderboard() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
        case .providerAppleShowAchievements:
            ProviderApple.shared.showAchievements { (result) in
                let log = """
                === ProviderApple.shared.showAchievements() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
        case .providerAppleLoadAchievements:
            ProviderApple.shared.loadAchievements { (result, achievements) in
                let log = """
                === ProviderApple.shared.loadAchievements() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
        case .providerAppleResetAchievements:
            ProviderApple.shared.resetAchievements { (result) in
                let log = """
                === ProviderApple.shared.resetAchievements() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                //
            }
        default:
            preconditionFailure("Not supported api")
        }
    }
    
    private func showConflict() {
        if self.conflictPlayer != nil {
            let alertViewController = UIAlertController(title: "Conflict", message: "select", preferredStyle: .alert)
            
            let selectAction = UIAlertAction(title: "switch", style: .cancel) { _ in
                AuthV4Interface.selectConflict(self.conflictPlayer!.playerID, handler: { (result, playerInfo) in
                    let log = """
                    === AuthV4Interface.selectConflict() ===
                    
                    result - \(result)
                    playerInfo - \(playerInfo)
                    """
                    
                    self.logDelegate?.addLog(log: log)
                })
            }
            
            let resolveAction = UIAlertAction(title: "resolve", style: .default) { (result) in
                AuthV4Interface.resolveConflict({ (result) in
                    let log = """
                    === AuthV4Interface.resolveConflict() ===
                    
                    result - \(result)
                    """
                    
                    self.logDelegate?.addLog(log: log)
                })                
            }
            
            alertViewController.addAction(selectAction)
            alertViewController.addAction(resolveAction)
            
            self.present(alertViewController, animated: true, completion: nil)
        }else {
            Logger.d("Conflict info is nil.")
        }
    }
}

extension AuthV4ViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? AuthV4TableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
