//
//  PromotionViewController
//  HIVE_SDK_Sample
//
//  Created by jhjang on 04/04/2019.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum PromotionApi: String, CaseIterable {
    case promotionType = "Select Promotion Type"
    case showPromotion = "Show Promotion"
    case showCustomContents = "Show Custom Contents"
    case showOfferwall = "Show Offerwall"
    case showReview = "Show Review"
    case showNativeReview = "Show Native Review"
    case getViewInfo = "Get View Info"
    case getBadgeInfo = "Get Badge Info"
    case getBannerInfo = "Get Banner Info"
    case getAppInvitationData = "Get App Invitation Data"
    case setAdditionalInfo = "Set Additional Info"
    case setEngagementHandler = "Set Engagement Handler"
    case setEngagementReady = "Set Engagment Ready"
    case updatePromotionData = "Update Promotion Data"
}

enum PromotionType: String, CaseIterable {
    case news = "NEWS"
    case banner = "BANNER"
	case bannerLagacy = "BANNERLEGACY"
    case notice = "NOTICE"
    case customView = "CUSTOMVIEW"
    case customBoard = "CUSTOMBOARD"
    case spot = "SPOT"
    case direct = "DIRECT"
}

class PromotionViewController: BaseViewController {
    let apis: [PromotionApi] = PromotionApi.allCases
    var promotionType: PromotionType?
    var isForced: Bool = false
    @IBOutlet weak var tableView: UITableView!
    
    var promotionTypeActionSheet: UIAlertController {
        get {
            let actionSheet = UIAlertController(title: "Promotion Type", message: "Select promotion type", preferredStyle: .actionSheet)
            
            let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
            actionSheet.addAction(cancelAction)
            
            for type in PromotionType.allCases {
                let action  = UIAlertAction(title: type.rawValue, style: .default) { (action) in
                    self.promotionType = type
                }
                
                actionSheet.addAction(action)
            }
            
            return actionSheet
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: PromotionApi) {
        switch api {
        case .promotionType:
            self.present(promotionTypeActionSheet, animated: true, completion: nil)
        case .showPromotion:
            if let promotionType = self.promotionType {
                let actionSheet = UIAlertController(title: "Enable forced", message: "Use forced", preferredStyle: .actionSheet)
                
                let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
                actionSheet.addAction(cancelAction)
                
                let trueAction = UIAlertAction(title: "force on", style: .default) { action in
                    self.isForced = true
                    self.showPromotion(promotionType: promotionType)
                }
                
                let falseAction = UIAlertAction(title: "force off", style: .default) { action in
                    self.isForced = false
                    self.showPromotion(promotionType: promotionType)
                }
                
                actionSheet.addAction(trueAction)
                actionSheet.addAction(falseAction)
                
                self.present(actionSheet, animated: true, completion: nil)
            }else {
                Logger.d("Select promotion type first.")
            }
        case .showCustomContents:
            Logger.d("showCustomContents")
            guard
                let promotionType = self.promotionType,
                promotionType != .news && promotionType != .notice && promotionType != .banner
            else {
                Logger.e("Select proper promotion type. current is - \(self.promotionType)")
                return
            }
            
            showAlertWith(title: "Contents Key", message: "Input - ") { (contents) in
                if  let customType = PromotionCustomType(stringValue: promotionType.rawValue),
                    let ct = contents {
                    PromotionInterface.showCustomContents(customType, contents: ct, handler: { (result, viewResultType) in
                        let log = """
                        === PromotionInterface.showCustomContents() ===
                        
                        result - \(result)
                        viewResultType - \(viewResultType)
                        """
                        
                        self.logDelegate?.addLog(log: log)
                    })
                }
            }
        case .showOfferwall:
            Logger.d("showOfferwall")
            PromotionInterface.showOfferwall { (result, viewResultType) in
                let log = """
                === PromotionInterface.showOfferwall() ===
                
                result - \(result)
                viewResultType - \(viewResultType)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showReview:
            Logger.d("showReview")
            PromotionInterface.showReview { (result, viewResultType) in
                let log = """
                === PromotionInterface.showReview() ===
                
                result - \(result)
                viewResultType - \(viewResultType)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showNativeReview:
            Logger.d("showNativeReview")
            PromotionInterface.showNativeReview()
        case .getViewInfo:
            Logger.d("getViewInfo")
            guard
                let promotionType = self.promotionType,
                promotionType != .news && promotionType != .notice && promotionType != .banner
                else {
                    Logger.e("Select proper promotion type. current is - \(self.promotionType)")
                    return
            }
            
            showAlertWith(title: "Contents Key", message: "Input - ") { (contents) in
                if  let customType = PromotionCustomType(stringValue: promotionType.rawValue),
                    let ct = contents {
                    
                    PromotionInterface.getViewInfo(customType, contents: ct, handler: { (result, infos) in
                        let log = """
                        === PromotionInterface.getViewInfo() ===
                        
                        result - \(result)
                        infos - \(infos)
                        """
                        
                        self.logDelegate?.addLog(log: log)
                    })
                }
            }
        case .getBadgeInfo:
            Logger.d("getBadgeInfo")
            
            PromotionInterface.getBadgeInfo { (result, badges) in
                let log = """
                === PromotionInterface.getBadgeInfo() ===
                
                result - \(result)
                badges - \(badges)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .getBannerInfo:
            Logger.d("getBannerInfo")

            let alertCampaignType = UIAlertController(title: "CampaignType", message: "select one", preferredStyle: .actionSheet)
            for campaignType in PromotionCampaignType.allCases {
                let actionCampaignType = UIAlertAction(title: campaignType.description, style: .default) { (campaignType) in

                    let alertBannerType = UIAlertController.init(title: "BannerType", message: "select one", preferredStyle: .actionSheet)
                    for bannerType in PromotionBannerType.allCases {
                        let actionBannerType = UIAlertAction(title: bannerType.description, style: .default) { (bannerType) in

                            self.showGetBannerInfo(campaignType: PromotionCampaignType(stringValue: campaignType.title!), bannerType: PromotionBannerType(stringValue: bannerType.title!))
                        }
                        alertBannerType.addAction(actionBannerType)
                    }
                    self.present(alertBannerType, animated: true, completion: nil)
                }
                alertCampaignType.addAction(actionCampaignType)
            }
            self.present(alertCampaignType, animated: true, completion: nil)
            

        case .getAppInvitationData:
            Logger.d("getAppInvitationData")
            
            PromotionInterface.getAppInvitationData { (result, invitationData) in
                let log = """
                === PromotionInterface.getAppInvitationData() ===
                
                result - \(result)
                invitationData - \(invitationData)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .setAdditionalInfo:
            Logger.d("setAdditionalInfo")
            showAlertWith(title: "Additional Info", message: "Input:") { (info) in
                PromotionInterface.setAdditionalInfo(info)
            }
        
        case .setEngagementHandler:
            PromotionInterface.setEngagementHandler { (result, eventType, state, params) in
                let log = """
                === PromotionInterface.setEngagementHandler() ===
                
                result - \(result)
                eventType - \(eventType)
                state - \(state)
                params - \(params?.description ?? "")
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .setEngagementReady:
            let result = PromotionInterface.setEngagementReady(true).isSuccess()
            let log = """
            === PromotionInterface.setEngagementReady() ===
            
            result - \(result)
            """
            
            self.logDelegate?.addLog(log: log)
        case .updatePromotionData:
            PromotionInterface.updatePromotionData()
            let log = """
            === PromotionInterface.updatePromotionData() ===
            """
        default:
            preconditionFailure("Not supported api")
        }
    }
    
    private func showAlertWith(title: String, message: String, handler: @escaping (_ text: String?) -> ()) {
        let alertViewController = UIAlertController(title: title, message: message, preferredStyle: .alert)

        alertViewController.addTextField { (textField) in
        }
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alertViewController.addAction(cancelAction)
        let confirmAction = UIAlertAction(title: "Confirm", style: .default) { (action) in
            if let textField = alertViewController.textFields?.first,
                let additionalInfo = textField.text {
                handler(additionalInfo)
            }else {
                Logger.e("[Promotion Sample] Text is empty or nil.")
                handler(nil)
            }
            
        }
        alertViewController.addAction(confirmAction)
        
        self.present(alertViewController, animated:true, completion: nil)

//        alert.addTextField(configurationHandler: configurationTextField)
//        alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler:nil))
//        alert.addAction(UIAlertAction(title: "Ok", style: .default, handler:{ (UIAlertAction) in
//            print("User click Ok button")
//        }))
//        self.present(alert, animated: true, completion: nil)
    }
    
    private func showPromotion(promotionType: PromotionType) {
        guard
            let viewType = PromotionViewType(stringValue: promotionType.rawValue)
        else {
            preconditionFailure("Invalid view type")
        }
        PromotionInterface.showPromotion(viewType, isForced: isForced) { (result, viewResultType) in
            let log = """
            === PromotionInterface.showPromotion() ===
            
            result - \(result)
            viewResultType - \(viewResultType)
            """
            
            self.logDelegate?.addLog(log: log)
        }
    }

    private func showGetBannerInfo(campaignType: PromotionCampaignType, bannerType: PromotionBannerType) {
        PromotionInterface.getBannerInfo(campaignType, bannerType: bannerType) { (result, bannerInfos) in
            var log = """
            === PromotionInterface.getBannerInfo() ===
            result - \(result)
            """

            guard let bannerInfos = bannerInfos
            else {
                Logger.d("[PromotionViewController] bannerInfos nil")
                return
            }

            for banner in bannerInfos {
                log.append(contentsOf: "\(banner.description)\n\n")
            }

            self.logDelegate?.addLog(log: log)

            if let promotionBannerViewController = PromotionBannerViewController(data: bannerInfos, completeHandler: { (result2, msg) in
                self.logDelegate?.addLog(log: msg)
            }) {
                promotionBannerViewController.modalPresentationStyle = .overFullScreen
                self.present(promotionBannerViewController, animated: false, completion: nil)
            } else {
                self.logDelegate?.addLog(log: "[PromotionViewController] promotionBannerViewController is nil")
            }

        }
    }
}

extension PromotionViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? PromotionTableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
